# ---- [net_io_gui.py ] -------------------------------------------------------
#
#       11.11.12

import _thread
import time
import tkinter      as tk
import net_io_tools as ni
import bargraph     as bg

# ------------------------------------------------------------------------------
# ein eigenständiger Thread, der in regelmäßigen Abständen den AVR-NET-IO anfunkt,
# die Daten abholt und die Anzeige im Hauptfenster aktualisiert.

def Timer_Thread():
    global telnet_error                 # Fehler beim Telnet-Zugriff
    
    while not telnet_error:
        ni.Open_Telnet()                # die Verbindung zum Server öffnen

        adc = ni.Get_Adcs()             # ADC's auslesen
                         
        adc_1.set(adc[0])               # die numerische Anzeige (neben dem Text "ADCx") aktualisieren
        adc_2.set(adc[1])
        adc_3.set(adc[2])
        adc_4.set(adc[3])
        
        bg1.draw(adc[0])                # die Anzeige der Bargraphen aktualisieren
        bg2.draw(adc[1])
        bg3.draw(adc[2])
        bg4.draw(adc[3])
        
        pinstate = ni.Get_Pin_State()   # Den Status der digitalen Eingänge einlesen
        
        in_1.set(pinstate[0])           # die Beschriftung der Labels ändern (0 oder 1)
        in_2.set(pinstate[1])
        in_3.set(pinstate[2])
        in_4.set(pinstate[3])

        r = ("raised", "sunken")        # und die Labels sunken/raised darstellen
                                        # Relief "sunken" wenn "1", Relief "raised" wenn "0"
        Label15.configure(relief = r[pinstate[0]] )
        Label16.configure(relief = r[pinstate[1]] )
        Label17.configure(relief = r[pinstate[2]] )
        Label18.configure(relief = r[pinstate[3]] )

        Refresh_Digi_Ports()            # liest den Status der Ausgänge ein

        ni.Close_Telnet()               # Verbindung zum Telnetserver kappen
        
        sleeptime = interval.get()      # das aktuell gewählte Intervall aus der Spinbox holen

        while sleeptime > 0:            # Wartezeit
            time.sleep(1)               # wartet immer nur für 1 Sekunde
            sleeptime -= 1              # 

            next_interval.set(sleeptime)# Anzeige der Wartezeit bis zur nächsten Messung aktualisieren

# -------------------------------------------
# Der "Beenden"- Button beendet die Anwendung
def ende():
    top.destroy()
    
# -------------------------------------------
# Wird aufgerufen, wenn der Button "I" angeklickt wurde
# gibt eine Messagebox mit den Hardwareinformationen des AVR-NET-IO aus

# Die Messagebox führt zu Programmende, wenn nicht über idle ausgeführt wird !
# Ausserdem zählt die Zeit nicht weiter ...
# def avrnetio_version():
#    tk.messagebox.showinfo("AVR-NET-IO Info", netio_message[1][0] + ' - '
#                     + netio_message[1][1] + ' - ' + netio_message[1][2])
#
#
# diese Variante funktioniert auch dann, wenn nicht aus Idle heraus gestartet wird
# und blockiert nicht die Ausführung des anderen Threads
def avrnetio_version():

    w = tk.Toplevel()       # ein unabhängiges Toplevel-Window öffnen
    w.resizable(width = False, height = False) # Größe nicht veränderbar
    w.title("AVR-NET-IO Info")
    f = tk.Label(w, width = 30, height = 5, font = ft,  text = (netio_message[1][0] + '\n' +
                     netio_message[1][1] + '\n' + netio_message[1][2]), anchor = "center")
    f.grid(column = 0, row = 0, sticky = "ewns")
    w.focus_set()           # focus auf das Fenster
    w.grab_set()            # alle Events werden nur nach w durchgereicht,
                            # das drunterliegende Fenster bleibt inaktiv, wird aber aktualisiert
    w.transient(top)        # das Fenster bleibt oberhalb von Top
#   w.wait_window(w)        # warten bis zum Beenden des Fensters, aber ...
                            # ... die Zeit zählt dann nicht weiter
    w.mainloop()            # warten bis zum Beenden des Fensters



# --- Funktionen der Buttons und Events der Entry's ---------------
#
# --- Wird aufgerufen beim Klicken des Buttons "send row" oder bei einem
#     RETURN in der Eingabezeile.
#     event = None ist erforderlich, weil beim Klicken eines Buttons
#     kein Parameter übergeben wird,
#     bei einem Enter dagegen wird das Event mit übergeben.
#     Die Funktionen müssen daher sowohl 0 als auch 1 Parameter entgegennehmen können.

def zeile_1_send(event = None):
    if LCD_OK == False: return

    ni.Open_Telnet()
    z = entry1.get()
    ni.Write_LCD(1,z)
    ni.Close_Telnet()
    
def zeile_2_send(event = None):
    if LCD_OK == False: return

    ni.Open_Telnet()
    z = entry2.get()
    ni.Write_LCD(2,z)         
    ni.Close_Telnet()
    
# --- Button "clr row"
#     Das Entry-Feld wird gelöscht und ein Lösch-Befehl an den Telnet-Server geschickt
#     Der Button übergibt keinen Parameter

def zeile_1_clr():
    entry1.set("")                  # Das Entry-Feld löschen
    if LCD_OK == False: return
    
    ni.Open_Telnet()
    ni.Clear_LCD(1)                 # Die Zeile 1 des LCD löschen
    ni.Close_Telnet()       

def zeile_2_clr():
    entry2.set("")
    if LCD_OK == False: return
    
    ni.Open_Telnet()
    ni.Clear_LCD(2)
    ni.Close_Telnet()


# --- Das Mainwindow -----------------------------------------------------
top = tk.Tk()
top.resizable(width = False, height = False)    # die Fenstergröße unveränderbar machen

# --- Funktionen und Variablen für die Fenster 

# --- Variablen, die den Status der Eingänge beinhalten ------------------
in_1  = tk.IntVar()
in_2  = tk.IntVar()
in_3  = tk.IntVar()
in_4  = tk.IntVar()

# --- Variablen, die den Status der Ausgänge beinhalten --------------
out_1 = tk.IntVar()
out_2 = tk.IntVar()
out_3 = tk.IntVar()
out_4 = tk.IntVar()
out_5 = tk.IntVar()
out_6 = tk.IntVar()
out_7 = tk.IntVar()
out_8 = tk.IntVar()

# --- Sendet immer dann die notwendigen Daten an den NET-IO raus,
# --- wenn eine Checkbox für einen Ausgang angewählt worden ist
def check_1():
    if telnet_error: return
    
    ni.Open_Telnet()
    ni.Set_Port_State(1, out_1.get())
    ni.Close_Telnet()
    
def check_2():
    if telnet_error: return

    ni.Open_Telnet()
    ni.Set_Port_State(2, out_2.get())
    ni.Close_Telnet()

def check_3():
    if telnet_error: return

    ni.Open_Telnet()
    ni.Set_Port_State(3, out_3.get())
    ni.Close_Telnet()

def check_4():
    if telnet_error: return

    ni.Open_Telnet()
    ni.Set_Port_State(4, out_4.get())
    ni.Close_Telnet()
    
def check_5(): 
    if telnet_error: return

    ni.Open_Telnet()
    ni.Set_Port_State(5, out_5.get())
    ni.Close_Telnet()

def check_6(): 
    if telnet_error: return

    ni.Open_Telnet()
    ni.Set_Port_State(6, out_6.get())
    ni.Close_Telnet()

def check_7(): 
    if telnet_error: return

    ni.Open_Telnet()
    ni.Set_Port_State(7, out_7.get())
    ni.Close_Telnet()

def check_8():
    if telnet_error: return

    ni.Open_Telnet()
    ni.Set_Port_State(8, out_8.get())
    ni.Close_Telnet()

# ------------------------------------------
# liest den Status aller Ports ein und weist den Wert
# den Variablen zu, die den Checkbutton steuern
def Refresh_Digi_Ports():
    global out_1, out_2, out_3, out_4, out_5, out_6, out_7, out_8   

    l = ni.Get_Port_State()
   
    out_1.set(l[0])
    out_2.set(l[1])
    out_3.set(l[2])
    out_4.set(l[3])
    out_5.set(l[4])
    out_6.set(l[5])
    out_7.set(l[6])
    out_8.set(l[7])
    
# --- Variablen für die Messwerte der ADC's --------------------
adc_1 = tk.IntVar()
adc_2 = tk.IntVar()
adc_3 = tk.IntVar()
adc_4 = tk.IntVar()

# --- Messintervall --------------------------
interval = tk.IntVar()
interval.set(10)                 # Startwert für das Messintervall
next_interval = tk.IntVar()
telnet_error = False

# --- Eingabetext für LCD-Zeile ------------------------
entry1 = tk.StringVar()
entry2 = tk.StringVar()
LCD_OK = False

# --- Fonts ---------------------------------------

ft = ("Courier",10,"bold" )     # Standard Schrift
#ft = ("LucidiaTypeWriter", 10, "bold")
fs = ("LucidiaTypeWriter", 8)   # Beschriftung der Frames und Bargraphen

# --- Jetzt folgt der Aufbau des Windows
#     Mit dem Grid-Manager wird die Seite analog wie eine HTML-Tabelle aufgebaut.
#     Im Hauptfenster Top werden Frames durch Angabe ihrer Position über row und column definiert
#     Innerhalb eines Frames werden die Widgets in gleicher Form in Tabellen-"Zellen" plaziert
#     Die Breite und Höhe einer jeden Zelle richtet sich nach dem größten Element, dass die
#     zugehörige row / column aufnehmen muss.


# --- Frame 1, für die 4 digitalen Eingänge -------

frame1 = tk.LabelFrame(top, padx = 5, pady = 5, text = "Read Digital Inputs", relief = "groove", font = fs)
frame1.grid(column = 0, row = 0, columnspan = 2, padx = 5, pady = 5)

Label11 = tk.Label(frame1, text = "Input 1:", anchor="e", width = 10, font = ft)
Label12 = tk.Label(frame1, text = "Input 2:", anchor="e", width = 10, font = ft)
Label13 = tk.Label(frame1, text = "Input 3:", anchor="e", width = 10, font = ft)
Label14 = tk.Label(frame1, text = "Input 4:", anchor="e", width = 10, font = ft)
                   
Label11.grid(column = 0, row = 0, padx = 5, pady = 3, sticky = "e")
Label12.grid(column = 0, row = 1, padx = 5, pady = 3, sticky = "e")
Label13.grid(column = 0, row = 2, padx = 5, pady = 3, sticky = "e")
Label14.grid(column = 0, row = 3, padx = 5, pady = 3, sticky = "e")

Label15 = tk.Label(frame1, textvariable = in_1, width = 4, font = ft) # hier noch kein relief setzen
Label16 = tk.Label(frame1, textvariable = in_2, width = 4, font = ft) # das erfolgt beim 1. Update
Label17 = tk.Label(frame1, textvariable = in_3, width = 4, font = ft) 
Label18 = tk.Label(frame1, textvariable = in_4, width = 4, font = ft) 

Label15.grid(column = 1, row = 0, padx = 5, pady = 3, sticky = "w")
Label16.grid(column = 1, row = 1, padx = 5, pady = 3, sticky = "w")
Label17.grid(column = 1, row = 2, padx = 5, pady = 3, sticky = "w")
Label18.grid(column = 1, row = 3, padx = 5, pady = 3, sticky = "w")


# --- FRAME 2, für die 10 digitalen Ausgänge --------

frame2 = tk.LabelFrame(top, padx = 5, pady = 5, text = "Set Outputs", relief = "groove", font = fs)
frame2.grid(column = 2, row = 0, columnspan = 2, padx = 5, pady = 5, sticky = "nsew")

Button21 = tk.Checkbutton(frame2, text = "Output 1", width = 12, anchor = "w", font = ft, relief = "groove", variable = out_1, command = check_1)
Button22 = tk.Checkbutton(frame2, text = "Output 2", width = 12, anchor = "w", font = ft, relief = "groove", variable = out_2, command = check_2)
Button23 = tk.Checkbutton(frame2, text = "Output 3", width = 12, anchor = "w", font = ft, relief = "groove", variable = out_3, command = check_3)
Button24 = tk.Checkbutton(frame2, text = "Output 4", width = 12, anchor = "w", font = ft, relief = "groove", variable = out_4, command = check_4)
Button25 = tk.Checkbutton(frame2, text = "Output 5", width = 12, anchor = "w", font = ft, relief = "groove", variable = out_5, command = check_5)
Button26 = tk.Checkbutton(frame2, text = "Output 6", width = 12, anchor = "w", font = ft, relief = "groove", variable = out_6, command = check_6)
Button27 = tk.Checkbutton(frame2, text = "Output 7", width = 12, anchor = "w", font = ft, relief = "groove", variable = out_7, command = check_7)
Button28 = tk.Checkbutton(frame2, text = "Output 8", width = 12, anchor = "w", font = ft, relief = "groove", variable = out_8, command = check_8)

Button21.grid(column = 2, row = 0, padx= 5, pady = 1, sticky = "w")
Button22.grid(column = 2, row = 1, padx= 5, pady = 1, sticky = "w")
Button23.grid(column = 2, row = 2, padx= 5, pady = 1, sticky = "w")
Button24.grid(column = 2, row = 3, padx= 5, pady = 1, sticky = "w")
Button25.grid(column = 3, row = 0, padx= 5, pady = 1, sticky = "w")
Button26.grid(column = 3, row = 1, padx= 5, pady = 1, sticky = "w")
Button27.grid(column = 3, row = 2, padx= 5, pady = 1, sticky = "w")
Button28.grid(column = 3, row = 3, padx= 5, pady = 1, sticky = "w")


# --- Frame 3, für die 4 analogen Eingänge mit digitaler und analoger Darstellung der Messwerte

frame3 = tk.LabelFrame(top, padx = 5, pady = 5, text = "Read 10-Bit Analog Inputs", relief = "groove", font = fs)
frame3.grid(column = 0, row = 2, columnspan = 4, padx = 5, pady = 5, sticky = "nsew")

Label31 = tk.Label(frame3, text = "ADC 1:", width = 10, anchor = "e", font = ft)
Label32 = tk.Label(frame3, text = "ADC 2:", width = 10, anchor = "e", font = ft)
Label33 = tk.Label(frame3, text = "ADC 3:", width = 10, anchor = "e", font = ft)
Label34 = tk.Label(frame3, text = "ADC 4:", width = 10, anchor = "e", font = ft)

Label31.grid(column = 0, row = 0, padx= 5, pady = 3, sticky = "e")
Label32.grid(column = 0, row = 1, padx= 5, pady = 3, sticky = "e")
Label33.grid(column = 0, row = 2, padx= 5, pady = 3, sticky = "e")
Label34.grid(column = 0, row = 3, padx= 5, pady = 3, sticky = "e")

Label35 = tk.Label(frame3, textvariable = adc_1, width = 5, font = ft, relief = "groove", anchor ="center")
Label36 = tk.Label(frame3, textvariable = adc_2, width = 5, font = ft, relief = "groove", anchor ="center")
Label37 = tk.Label(frame3, textvariable = adc_3, width = 5, font = ft, relief = "groove", anchor ="center")
Label38 = tk.Label(frame3, textvariable = adc_4, width = 5, font = ft, relief = "groove", anchor ="center")

Label35.grid(column = 1, row = 0, padx= 5, pady = 3, sticky = "w")
Label36.grid(column = 1, row = 1, padx= 5, pady = 3, sticky = "w")
Label37.grid(column = 1, row = 2, padx= 5, pady = 3, sticky = "w")
Label38.grid(column = 1, row = 3, padx= 5, pady = 3, sticky = "w")

Space39 = tk.Label(frame3, width = 0)  # Platz-/Abstandshalter
Space39.grid(column = 4, row = 0, padx= 3, pady = 3, sticky = "ew")

bg1 = bg.BarGraph(frame3, width = 256, end_value = 1023, font = ft, label = (5, "Va ", " mV"))
bg2 = bg.BarGraph(frame3, width = 256, end_value = 1023, font = ft, label = (5, "Vb ", " mV"))
bg3 = bg.BarGraph(frame3, width = 256, end_value = 1023, font = ft, label = (5, "Vc ", " mV"))
bg4 = bg.BarGraph(frame3, width = 256, end_value = 1023, font = ft, label = (5, "Vd ", " mV"))


bg1.grid(column = 5, row = 0, padx = 5, pady = 3, sticky = "ne")
bg2.grid(column = 5, row = 1, padx = 5, pady = 3, sticky = "ne")
bg3.grid(column = 5, row = 2, padx = 5, pady = 3, sticky = "ne")
bg4.grid(column = 5, row = 3, padx = 5, pady = 3, sticky = "ne")

# --- Frame 4 - LCD-Eingabezeilen und Buttons zum Senden / Löschen ------

frame4 = tk.LabelFrame(top, padx = 5, pady = 5, text = "Set LCD Output", relief = "groove", font = fs)
frame4.grid(column = 0, row = 3, columnspan = 4, padx = 5, pady = 5, sticky = "nsew")

Label41 = tk.Label(frame4, text = "LCD row 1:", width = 10, anchor = "e", font = ft)
Label42 = tk.Label(frame4, text = "LCD row 2:", width = 10, anchor = "e", font = ft)

Label41.grid(column = 0, row = 0, padx= 5, pady = 3, sticky = "e")
Label42.grid(column = 0, row = 1, padx= 5, pady = 3, sticky = "e")

Entry41 = tk.Entry(frame4, textvariable = entry1, width = 16, font = ft)
Entry42 = tk.Entry(frame4, textvariable = entry2, width = 16, font = ft)

Entry41.grid(column = 1, row = 0, padx= 5, pady = 3, sticky = "w")
Entry42.grid(column = 1, row = 1, padx= 5, pady = 3, sticky = "w")

Entry41.bind(sequence ='<Return>', func= zeile_1_send)
Entry42.bind(sequence ='<Return>', func= zeile_2_send)

Button41 = tk.Button(frame4, text = "send row", font = ft, width = 10, anchor = "center", command = zeile_1_send)
Button42 = tk.Button(frame4, text = "send row", font = ft, width = 10, anchor = "center", command = zeile_2_send)

Button41.grid(column = 3, row = 0, padx= 5, pady = 3, sticky = "e")
Button42.grid(column = 3, row = 1, padx= 5, pady = 3, sticky = "e")

Button411 = tk.Button(frame4, text = "clr row", font = ft, width = 10, anchor = "center", command = zeile_1_clr)
Button412 = tk.Button(frame4, text = "clr row", font = ft, width = 10, anchor = "center", command = zeile_2_clr)

Button411.grid(column = 4, row = 0, padx= 4, pady = 3, sticky = "e")
Button412.grid(column = 4, row = 1, padx= 4, pady = 3, sticky = "e")

# -- Frame 5 ---- Auswahl Messintervall, Info und Beenden ---------------

frame5 = tk.LabelFrame(top, padx = 5, pady = 5, relief = "groove", font = fs)
frame5.grid(column = 0, row = 4, columnspan = 4, rowspan = 1, padx = 5, pady = 5, sticky = "new")

Label51 = tk.Label(frame5, text = "Intervall:" , padx = 3, pady = 3, width = 10, font = ft, anchor = "e")
Label51.grid(column = 0, row = 0, padx= 4, pady = 5, sticky = "w")

spinbtn = tk.Spinbox(frame5, font = ft, width = 4, textvariable = interval, from_ = 1, to = 120, state = "readonly")
spinbtn.grid(column = 1, row = 0, padx = 4, pady = 3, sticky = "ew")

Button51 = tk.Label(frame5, text = "[Sek] next in :",  font = ft, width = 14, anchor ="w")
Button51.grid(column = 2, row = 0, padx= 3, pady = 3, sticky = "ew")

Label51 = tk.Label(frame5, textvariable = next_interval,  font = ft, width = 3, anchor ="center")
Label51.grid(column = 3, row = 0, padx= 1, pady = 3, sticky = "ew")

Button52 = tk.Button(frame5, text = "I", font = ft, width = 1, anchor = "center", command = avrnetio_version)
Button52.grid(column = 4, row = 0, padx= 5, pady = 4, sticky = "e")

Button53 = tk.Button(frame5, text = "Quit",  font = ft, width = 10, anchor = "center", command = ende)
Button53.grid(column = 5, row = 0, padx= 5, pady = 3, sticky = "e")

# --- nun ist das Fenster fertig

netio_message = ni.Start_Telnet()       # schauen ob die Verbindung zum Server steht

if len(netio_message[1][2]) == 0:
    telnet_error = True
    tk.messagebox.showerror(message = "AVR-NET-IO nicht gefunden")
       
top.title("AVR-NET-IO - " + netio_message[0]) # die Verbindungsdaten in Fenstertitel aufnehmen

if not telnet_error:
    ni.Open_Telnet()                    # die Verbindung zum Server öffnen, um das LCD auf Präsenz zu testen

    if ni.Init_LCD():
        LCD_OK = True
    
    else:                               # Bei fehlendem LCD treten nach der erfolglosen Initialsierung Probleme auf
        ni.Close_Telnet()               # Als Workaround die Verbindung schließen
                                        # um Fehlerhafte Daten in Get_Port_State() zu vermeiden
                    
        entry1.set("no LCD found")      # Eingabefelder und Buttons bei fehlendem LCD deaktivieren
        entry2.set("no LCD found")
        Entry41.configure(state  = "disabled")
        Entry42.configure(state  = "disabled")
        Button41.configure(text  = "---")
        Button42.configure(text  = "---")
        Button411.configure(text = "---")
        Button412.configure(text = "---") 

        ni.Open_Telnet()                # Workaround - Verbindung wieder öffnen
    
    Refresh_Digi_Ports()                # den Status die digitalen Ausgänge auslesen,
                                        # und die Anzeige aktualisieren

    ni.Close_Telnet()                   # die Verbindung zum Server schließen

_thread.start_new_thread(Timer_Thread,())# den Thread starten, der die Messungen anstößt und die Messwerte aktualisiert

top.mainloop()

# --- [ eof ] ----------------------------------------------------------------------------
